@@ -103,6 +103,7 @@ class OperatorsPseudoSpectral2D(_Operators, OperatorBase):
             "Ly": 8,
             "truncation_shape": "cubic",
             "NO_SHEAR_MODES": False,
+            "NO_KY0": False,
         }
         params._set_child("oper", attribs=attribs)
 
@@ -172,10 +173,22 @@ class OperatorsPseudoSpectral2D(_Operators, OperatorBase):
         else:
             if NO_SHEAR_MODES:
                 COND_NOSHEAR = abs(self.KX) == 0.0
-                where_dealiased = np.logical_or(
-                    COND_NOSHEAR, self.where_dealiased
+                self.where_dealiased = np.array(
+                    np.logical_or(COND_NOSHEAR, self.where_dealiased),
+                    dtype=np.uint8,
+                )
+
+        try:
+            NO_KY0 = self.params.oper.NO_KY0
+        except AttributeError:
+            pass
+        else:
+            if NO_KY0:
+                COND_NO_KY0 = abs(self.KY) == 0.0
+                self.where_dealiased = np.array(
+                    np.logical_or(COND_NO_KY0, self.where_dealiased),
+                    dtype=np.uint8,
                 )
-                self.where_dealiased = np.array(where_dealiased, dtype=np.uint8)
 
     def get_region_multiple_aliases(self):
         aliases_x = abs(self.KX) >= 2 / 3 * self.deltakx * self.nx / 2
