@@ -21,7 +21,6 @@ Internal API
 
 """
 
-import inspect
 import os
 import time
 from copy import deepcopy as _deepcopy
@@ -39,6 +38,7 @@ import numpy as _np
 from fluiddyn.io.redirect_stdout import stdout_redirected
 from fluiddyn.util import mpi
 from fluiddyn.util.util import get_memory_usage
+
 from fluidsim_core import loader
 
 from fluidsim import path_dir_results
@@ -51,6 +51,7 @@ from fluidsim.base.params import (
     merge_params,
 )
 from fluidsim.base.solvers.info_base import create_info_simul
+from fluidsim.extend_simul import _extend_simul_class_from_path
 
 from .output import save_file
 
@@ -86,7 +87,7 @@ def available_solver_keys():
 def get_dim_from_solver_key(key):
     """Try to guess the dimension from the solver key (via the operator name)."""
     cls = import_simul_class_from_key(key)
-    info = cls.InfoSolver()
+    info = cls.InfoSolver(only_root=True)
     class_name = info.classes.Operators.class_name
 
     # special cases:
@@ -112,7 +113,7 @@ def pathdir_from_namedir(name_dir: Union[str, Path, None] = None):
 
     """
     if name_dir is None:
-        return os.getcwd()
+        return Path.cwd()
 
     if not isinstance(name_dir, Path):
         name_dir = Path(name_dir)
@@ -142,6 +143,17 @@ class ModulesSolvers(dict):
 def name_file_from_time_approx(path_dir, t_approx=None):
     """Return the file name whose time is the closest to the given time.
 
+    Parameters
+    ----------
+
+    path_dir: Path or str
+
+      Path of the directory of the simulation.
+
+    t_approx : number or "last" (optional)
+
+      Approximate time of the file to be loaded.
+
     .. todo::
 
         Can be elegantly implemented using regex as done in
@@ -151,11 +163,16 @@ def name_file_from_time_approx(path_dir, t_approx=None):
     if not isinstance(path_dir, Path):
         path_dir = Path(path_dir)
 
-    path_files = sorted(tuple(path_dir.glob("state_phys_t*")))
+    path_files = sorted(path_dir.glob("state_phys_t*"))
+
     nb_files = len(path_files)
     if nb_files == 0 and mpi.rank == 0:
         raise ValueError("No state file in the dir\n" + str(path_dir))
 
+    if t_approx is None:
+        # should be the last one but not 100% sure
+        return path_files[-1].name
+
     name_files = [path.name for path in path_files]
     if "state_phys_t=" in name_files[0]:
         ind_start_time = len("state_phys_t=")
@@ -168,7 +185,7 @@ def name_file_from_time_approx(path_dir, t_approx=None):
         if "_" in tmp:
             tmp = tmp[: tmp.index("_")]
         times[ii] = float(tmp)
-    if t_approx is None:
+    if t_approx == "last":
         t_approx = times.max()
     i_file = abs(times - t_approx).argmin()
     name_file = path_files[i_file].name
@@ -232,8 +249,19 @@ def load_sim_for_plot(
 
     fix_old_params(params)
 
+    try:
+        path_file = Path(path_dir) / name_file_from_time_approx(path_dir)
+    except ValueError:
+        # no state_phys file to extend the simul class. Let's try with info_solver.xml
+        path_file = Path(path_dir) / "info_solver.xml"
+
+    if path_file.exists():
+        Simul = _extend_simul_class_from_path(solver.Simul, path_file)
+    else:
+        Simul = solver.Simul
+
     with stdout_redirected(hide_stdout):
-        sim = solver.Simul(params)
+        sim = Simul(params)
     return sim
 
 
@@ -245,7 +273,7 @@ def _import_solver_from_path(path_dir):
 
 def load_state_phys_file(
     name_dir=None,
-    t_approx=None,
+    t_approx="last",
     modif_save_params=True,
     merge_missing_params=False,
     init_with_initialized_state=True,
@@ -314,7 +342,7 @@ def load_state_phys_file(
     return sim
 
 
-def load_for_restart(name_dir=None, t_approx=None, merge_missing_params=False):
+def load_for_restart(name_dir=None, t_approx="last", merge_missing_params=False):
     """Load params and Simul for a restart.
 
     >>> params, Simul = load_for_restart(name_dir)
@@ -342,15 +370,18 @@ def load_for_restart(name_dir=None, t_approx=None, merge_missing_params=False):
 
     path_dir = pathdir_from_namedir(name_dir)
     solver = _import_solver_from_path(path_dir)
+    Simul = solver.Simul
 
     # choose the file with the time closer to t_approx
     name_file = name_file_from_time_approx(path_dir, t_approx)
     path_file = os.path.join(path_dir, name_file)
 
+    Simul = _extend_simul_class_from_path(Simul, path_file)
+
     if merge_missing_params:
         # this has to be done by all processes otherwise there is a problem
         # with Transonic (see https://foss.heptapod.net/fluiddyn/fluidsim/issues/26)
-        default_params = solver.Simul.create_default_params()
+        default_params = Simul.create_default_params()
 
     if mpi.rank > 0:
         params = None
@@ -376,7 +407,7 @@ def load_for_restart(name_dir=None, t_approx=None, merge_missing_params=False):
     if mpi.nb_proc > 1:
         params = mpi.comm.bcast(params, root=0)
 
-    return params, solver.Simul
+    return params, Simul
 
 
 def modif_resolution_all_dir(t_approx=None, coef_modif_resol=2, dir_base=None):
@@ -385,14 +416,10 @@ def modif_resolution_all_dir(t_approx=None, coef_modif_resol=2, dir_base=None):
 
 
 def modif_resolution_from_dir(
-    name_dir=None, t_approx=None, coef_modif_resol=2, PLOT=True
+    name_dir=None, t_approx="last", coef_modif_resol=2, PLOT=True
 ):
     """Save a file with a modified resolution."""
 
-    path_dir = pathdir_from_namedir(name_dir)
-
-    solver = _import_solver_from_path(path_dir)
-
     sim = load_state_phys_file(name_dir, t_approx)
 
     params2 = _deepcopy(sim.params)
@@ -409,18 +436,16 @@ def modif_resolution_from_dir(
 
     params2.init_fields.type = "from_simul"
 
-    sim2 = solver.Simul(params2)
+    sim2 = sim.__class__(params2)
     sim2.init_fields.get_state_from_simul(sim)
 
-    print(sim2.params.path_run)
-
     oper_new = sim2.params.oper
     if dimension == 3:
         dir_new_file = f"/State_phys_{oper_new.nx}x{oper_new.ny}x{oper_new.nz}"
     else:
         dir_new_file = f"/State_phys_{oper_new.nx}x{oper_new.ny}"
 
-    sim2.output.path_run = str(path_dir) + dir_new_file
+    sim2.output.path_run = str(sim.output.path_run) + dir_new_file
     print("Save file in directory\n" + sim2.output.path_run)
     sim2.output.phys_fields.save(particular_attr="modif_resolution")
 
@@ -504,7 +529,7 @@ class StatePhysLike:
 
 
 def modif_resolution_from_dir_memory_efficient(
-    name_dir=None, t_approx=None, coef_modif_resol=2
+    name_dir=None, t_approx="last", coef_modif_resol=2
 ):
     """Save a file with a modified resolution.
 
@@ -524,12 +549,11 @@ def modif_resolution_from_dir_memory_efficient(
     path_file = path_dir / name_file
     print(f"Changing resolution of the state contained in\n{path_file}")
 
-    Simul = solver.Simul
+    Simul = _extend_simul_class_from_path(solver.Simul, path_file)
     try:
         info_solver = Simul.info_solver
     except AttributeError:
         info_solver = Simul.InfoSolver()
-        info_solver.complete_with_classes()
 
     with h5py.File(path_file, "r") as h5file:
         params = Parameters(hdf5_object=h5file["/info_simul/params"])
